@@ -38,8 +38,11 @@ namespace ph = std::placeholders;
 
 namespace {
 
+// Test fixture used to distinguish tests that require root privileges.
+struct RootPingCheckMgr : PingCheckMgr {};
+
 // Sanity check the basics for production class, PingCheckMgr, single-threaded mode.
-TEST(PingCheckMgr, basicsST) {
+TEST(RootPingCheckMgr, basicsST) {
     SKIP_IF(IOServiceTest::notRoot());
     MultiThreadingMgr::instance().setMode(false);
 
@@ -112,7 +115,7 @@ TEST(PingCheckMgr, basicsST) {
 
 // Sanity check the basics for production class, PingCheckMgr. Bulk of testing
 // is done with test derivation, TestPingCheckMgr.
-TEST(PingCheckMgr, basicsMT) {
+TEST(RootPingCheckMgr, basicsMT) {
     SKIP_IF(IOServiceTest::notRoot());
     MultiThreadingTest mt;
 
@@ -752,7 +755,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create manager with thread-pool size of 3, min_echos 2, reply_timeout 250 ms.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         ASSERT_NO_THROW_LOG(createMgr(3, 2, 250));
         ASSERT_TRUE(mgr_);
 
@@ -839,7 +842,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create a paused manager. 3 threads, 2 echos, 250 ms timeout.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         createMgr(3, 2, 250, true);
 
         // Calling nextToSend() should return false.
@@ -901,7 +904,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create a paused manager. 3 threads, 2 echos, 500 ms timeout.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         createMgr(3, 2, 500, true);
 
         // Should not have an expiration time, timer should not be running.
@@ -1009,7 +1012,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create a paused manager. 3 threads, 2 echos, 500 ms timeout.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         createMgr(3, 2, 500, true);
 
         // Start a ping for an address so we have a context.
@@ -1069,7 +1072,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create a paused manager. 3 threads, 2 echos, 500 ms timeout.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         createMgr(3, 2, 500, true);
 
         // Install a post reply received callback to stop the test if we're done.
@@ -1140,7 +1143,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create a paused manager. 3 threads, 2 echos, 500 ms timeout.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         createMgr(3, 2, 500, true);
 
         // Install a post reply received callback to stop the test if we're done.
@@ -1205,7 +1208,7 @@ public:
         SKIP_IF(notRoot());
 
         // Create a paused manager. 3 threads, 1 echos, 250 ms timeout.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         createMgr(3, 1, 250, true);
 
         // Start four ping checks, then stage them so:
@@ -1290,7 +1293,7 @@ public:
         // Create manager with thread-pool size of 3, min_echos 1,
         // reply_timeout 1000 milliseconds.  Larger time out for this test
         // avoids sporadic expirations which leads to unaccounted for UNPARKs.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         ASSERT_NO_THROW_LOG(createMgr(3, 1, 1000));
         ASSERT_TRUE(mgr_);
 
@@ -1341,7 +1344,7 @@ public:
         // Create manager with thread-pool size of 3, min_echos 1,
         // reply_timeout 1000 milliseconds.  Larger time out for this test
         // avoids sporadic expirations which leads to unaccounted for UNPARKs.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         ASSERT_NO_THROW_LOG(createMgr(3, 1, 1000));
         ASSERT_TRUE(mgr_);
 
@@ -1436,7 +1439,7 @@ public:
 
         // Create manager with thread-pool size of 3, min_echos 1,
         // reply_timeout 250 milliseconds.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         ASSERT_NO_THROW_LOG(createMgr(3, 1, 250));
         ASSERT_TRUE(mgr_);
 
@@ -1483,9 +1486,12 @@ public:
     void testShouldPingTest() {
         SKIP_IF(notRoot());
 
+        // Mnemonic local.
+        ConstHostPtr empty_host;
+
         // Create manager with thread-pool size of 3, min_echos 1,
         // reply_timeout 250 milliseconds.
-        // ST mode should ingore requested thread number.
+        // ST mode should ignore requested thread number.
         ASSERT_NO_THROW_LOG(createMgr(3, 1, 250));
         ASSERT_TRUE(mgr_);
 
@@ -1503,7 +1509,7 @@ public:
 
         // Ping checking enabled, no old lease, channel doesn't exist, should return CONTINUE.
         ASSERT_TRUE(config->getEnablePingCheck());
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, config));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_CONTINUE);
 
         // Start the manager, then pause it.  This lets us start pings without
@@ -1513,34 +1519,59 @@ public:
 
         // Ping checking disabled, no old lease, should return CONTINUE.
         config->setEnablePingCheck(false);
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, config));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_CONTINUE);
 
         // Ping checking enabled, no old lease, should return PARK.
         config->setEnablePingCheck(true);
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, config));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_PARK);
 
         // Make an old lease based on the first lease.
         time_t now = time(0);
         Lease4Ptr old_lease(new Lease4(*(lqp1.lease_)));
 
-        // Prior lease belonging to the same client with cltt greater than ping-cltt-secs
+        // Expired prior lease belonging to the same client with cltt greater than ping-cltt-secs
         // should return PARK.
         old_lease->cltt_ = now - config->getPingClttSecs() * 2;
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, config));
+        ASSERT_TRUE(old_lease->expired());
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, 
+                            empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_PARK);
 
-        // Prior lease belonging to the same client but with cltt less than ping-cltt-secs
+        // Lease with host with no address reserved should ping.
+        ConstHostPtr host;
+        ASSERT_NO_THROW((host.reset(new Host("01:02:03:04:05:06", "hw-address", 
+                                             SubnetID(0), SubnetID(0), IOAddress("0.0.0.0")))));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, 
+                                                      host, config));
+        EXPECT_EQ(status, CalloutHandle::NEXT_STEP_PARK);
+
+        // Lease with host with address reserved should not ping.
+        ASSERT_NO_THROW((host.reset(new Host("01:02:03:04:05:06", "hw-address", 
+                                            SubnetID(0), SubnetID(0), lqp1.lease_->addr_))));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, 
+                                                      host, config));
+        EXPECT_EQ(status, CalloutHandle::NEXT_STEP_CONTINUE);
+
+        // Expired prior lease belonging to the same client but with cltt less than ping-cltt-secs
         // should return CONTINUE.
         old_lease->cltt_ = now - config->getPingClttSecs() / 2;
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, config));
+        ASSERT_TRUE(old_lease->expired());
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, empty_host, config));
+        EXPECT_EQ(status, CalloutHandle::NEXT_STEP_CONTINUE);
+
+        // An active prior lease belonging to the same client should return continue.
+        old_lease->cltt_ = now - config->getPingClttSecs() * 2;
+        old_lease->valid_lft_ = 86400;
+        ASSERT_FALSE(old_lease->expired());
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_CONTINUE);
 
         // Prior lease belonging to a different client, should return PARK.
         const uint8_t id2[] = { 0x35, 0x36, 0x37, 0x34 };
         old_lease->client_id_.reset(new ClientId(id2, sizeof(id2)));
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, config));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, old_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_PARK);
 
         // Now let's start a ping for the lease-query pair.
@@ -1551,7 +1582,7 @@ public:
         lqp2.lease_->client_id_ = old_lease->client_id_;
 
         // Trying to start a ping for an address already being checked should return DROP.
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp2.lease_, lqp2.query_, empty_lease, config));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp2.lease_, lqp2.query_, empty_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_DROP);
 
         // Stop the mgr.
@@ -1559,7 +1590,7 @@ public:
 
         // Ping checking enabled, no old lease, channel isn't open, should return CONTINUE.
         ASSERT_TRUE(config->getEnablePingCheck());
-        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, config));
+        ASSERT_NO_THROW_LOG(status = mgr_->shouldPing(lqp1.lease_, lqp1.query_, empty_lease, empty_host, config));
         EXPECT_EQ(status, CalloutHandle::NEXT_STEP_CONTINUE);
     }
 
@@ -1749,110 +1780,114 @@ public:
     LeaseQueryPairs frees_;
 };
 
-TEST_F(PingCheckMgrTest, operationalBasicsST) {
+// Test fixture used to distinguish tests that require root privileges.
+struct RootPingCheckMgrTest : PingCheckMgrTest {};
+
+
+TEST_F(RootPingCheckMgrTest, operationalBasicsST) {
     testOperationalBasics(0);
 }
 
-TEST_F(PingCheckMgrTest, operationalBasicsMT) {
+TEST_F(RootPingCheckMgrTest, operationalBasicsMT) {
     MultiThreadingTest mt;
     testOperationalBasics(3);
 }
 
-TEST_F(PingCheckMgrTest, startPingST) {
+TEST_F(RootPingCheckMgrTest, startPingST) {
     testStartPing();
 }
 
-TEST_F(PingCheckMgrTest, startPingMT) {
+TEST_F(RootPingCheckMgrTest, startPingMT) {
     MultiThreadingTest mt;
     testStartPing();
 }
 
-TEST_F(PingCheckMgrTest, nextToSendST) {
+TEST_F(RootPingCheckMgrTest, nextToSendST) {
     testNextToSend();
 }
 
-TEST_F(PingCheckMgrTest, nextToSendMT) {
+TEST_F(RootPingCheckMgrTest, nextToSendMT) {
     MultiThreadingTest mt;
     testNextToSend();
 }
 
-TEST_F(PingCheckMgrTest, setNextExpirationST) {
+TEST_F(RootPingCheckMgrTest, setNextExpirationST) {
     testSetNextExpiration();
 }
 
-TEST_F(PingCheckMgrTest, setNextExpirationMT) {
+TEST_F(RootPingCheckMgrTest, setNextExpirationMT) {
     MultiThreadingTest mt;
     testSetNextExpiration();
 }
 
-TEST_F(PingCheckMgrTest, sendCompletedST) {
+TEST_F(RootPingCheckMgrTest, sendCompletedST) {
     testSendCompleted();
 }
 
-TEST_F(PingCheckMgrTest, sendCompletedMT) {
+TEST_F(RootPingCheckMgrTest, sendCompletedMT) {
     MultiThreadingTest mt;
     testSendCompleted();
 }
 
-TEST_F(PingCheckMgrTest, replyReceivedForEchoReplyST) {
+TEST_F(RootPingCheckMgrTest, replyReceivedForEchoReplyST) {
     testReplyReceivedForEchoReply();
 }
 
-TEST_F(PingCheckMgrTest, replyReceivedForEchoReplyMT) {
+TEST_F(RootPingCheckMgrTest, replyReceivedForEchoReplyMT) {
     MultiThreadingTest mt;
     testReplyReceivedForEchoReply();
 }
 
-TEST_F(PingCheckMgrTest, replyReceivedForTargetUnreachableST) {
+TEST_F(RootPingCheckMgrTest, replyReceivedForTargetUnreachableST) {
     testReplyReceivedForTargetUnreachable();
 }
 
-TEST_F(PingCheckMgrTest, replyReceivedForTargetUnreachableMT) {
+TEST_F(RootPingCheckMgrTest, replyReceivedForTargetUnreachableMT) {
     MultiThreadingTest mt;
     testReplyReceivedForTargetUnreachable();
 }
 
-TEST_F(PingCheckMgrTest, expirationProcessingST) {
+TEST_F(RootPingCheckMgrTest, expirationProcessingST) {
     testExpirationProcessing();
 }
 
-TEST_F(PingCheckMgrTest, expirationProcessingMT) {
+TEST_F(RootPingCheckMgrTest, expirationProcessingMT) {
     MultiThreadingTest mt;
     testExpirationProcessing();
 }
 
-TEST_F(PingCheckMgrTest, multiplePingsWithReplyST) {
+TEST_F(RootPingCheckMgrTest, multiplePingsWithReplyST) {
     testMultiplePingsWithReply();
 }
 
-TEST_F(PingCheckMgrTest, multiplePingsWithReplyMT) {
+TEST_F(RootPingCheckMgrTest, multiplePingsWithReplyMT) {
     MultiThreadingTest mt;
     testMultiplePingsWithReply();
 }
 
-TEST_F(PingCheckMgrTest, multiplePingsWithReplyAndPauseST) {
+TEST_F(RootPingCheckMgrTest, multiplePingsWithReplyAndPauseST) {
     testMultiplePingsWithReplyAndPause();
 }
 
-TEST_F(PingCheckMgrTest, multiplePingsWithReplyAndPauseMT) {
+TEST_F(RootPingCheckMgrTest, multiplePingsWithReplyAndPauseMT) {
     MultiThreadingTest mt;
     testMultiplePingsWithReplyAndPause();
 }
 
-TEST_F(PingCheckMgrTest, sendCompletedSendFailedST) {
+TEST_F(RootPingCheckMgrTest, sendCompletedSendFailedST) {
     testSendCompletedSendFailed();
 }
 
-TEST_F(PingCheckMgrTest, sendCompletedSendFailedMT) {
+TEST_F(RootPingCheckMgrTest, sendCompletedSendFailedMT) {
     MultiThreadingTest mt;
     testSendCompletedSendFailed();
 }
 
-TEST_F(PingCheckMgrTest, shouldPingST) {
+TEST_F(RootPingCheckMgrTest, shouldPingST) {
     testShouldPingTest();
 }
 
-TEST_F(PingCheckMgrTest, shouldPingMT) {
+TEST_F(RootPingCheckMgrTest, shouldPingMT) {
     MultiThreadingTest mt;
     testShouldPingTest();
 }
@@ -1866,11 +1901,11 @@ TEST_F(PingCheckMgrTest, getScopedConfigMT) {
     testGetScopedConfig();
 }
 
-TEST_F(PingCheckMgrTest, checkSuspendedST) {
+TEST_F(RootPingCheckMgrTest, checkSuspendedST) {
     testCheckSuspended();
 }
 
-TEST_F(PingCheckMgrTest, checkSuspendedMT) {
+TEST_F(RootPingCheckMgrTest, checkSuspendedMT) {
     MultiThreadingTest mt;
     testCheckSuspended();
 }
